#include <nds.h>
#include <nds/arm9/console.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <ipc2.h>

#define INPUT_C

#include "input.h"
#include "screen.h"
#include "text.h"
#include "wheel.h"
#include "file.h"
#include "sound.h"
#include "playlist.h"

struct input_handle *currenthandle;
u8 input_hold;

static void menu_down(void);
static void menu_up(void);
static void menu_center(void);
static void menu_play(void);
static void menu_next(void);
static void menu_prev(void);
static void menu_stop(void);

static void nowplaying_back(void);
static void nowplaying_next(void);
static void nowplaying_prev(void);
static void nowplaying_playpause(void);

static void music_back(void);
static void music_up(void);
static void music_down(void);
static void music_center(void);

static void playlist_back(void);
static void playlist_up(void);
static void playlist_down(void);
static void playlist_center(void);

static void playlists_back(void);
static void playlists_up(void);
static void playlists_down(void);
static void playlists_center(void);

static enum screen_menu nowplaying_last_menu;
static u32 nowplaying_last_selected;
static u32 music_last_selected;
static enum screen_menu playlist_last_menu;
static u32 playlist_last_selected;
static u32 playlists_last_selected;
static u32 playlists_playlist;

static u8 input_abort;

static struct input_handle menu_input[] = {
	{ WHEEL, WHEEL_RIGHT, menu_down },
	{ KEY, KEY_DOWN, menu_down },
	{ WHEEL, WHEEL_LEFT, menu_up },
	{ KEY, KEY_UP, menu_up},
	{ WHEEL, WHEEL_CENTER, menu_center },
	{ KEY, KEY_RIGHT, menu_center },
//	{ KEY, KEY_B, menu_play },
//	{ KEY, KEY_A, menu_next },
//	{ KEY, KEY_Y, menu_prev },
//	{ KEY, KEY_X, menu_stop },
	{ END, 0, NULL }
};
struct input_handle *menu_input_p = menu_input;

static struct input_handle nowplaying_input[] = {
	{ WHEEL, WHEEL_MENU, nowplaying_back },
	{ KEY, KEY_LEFT, nowplaying_back },
	{ KEY, KEY_X, nowplaying_back },
	{ WHEEL, WHEEL_FORWARD, nowplaying_next },
	{ KEY, KEY_A, nowplaying_next },
	{ WHEEL, WHEEL_REWIND, nowplaying_prev },
	{ KEY, KEY_Y, nowplaying_prev },
	{ WHEEL, WHEEL_PLAY, nowplaying_playpause },
	{ KEY, KEY_B, nowplaying_playpause },
	{ END, 0, NULL }
};
struct input_handle *nowplaying_input_p = nowplaying_input;

static struct input_handle music_input[] = {
	{ WHEEL, WHEEL_MENU, music_back },
	{ KEY, KEY_LEFT, music_back },
	{ KEY, KEY_X, music_back },
	{ WHEEL, WHEEL_RIGHT, music_down },
	{ KEY, KEY_DOWN, music_down },
	{ WHEEL, WHEEL_LEFT, music_up },
	{ KEY, KEY_UP, music_up},
	{ WHEEL, WHEEL_CENTER, music_center },
	{ KEY, KEY_RIGHT, music_center },
	{ END, 0, NULL }
};
struct input_handle *music_input_p = music_input;

static struct input_handle playlist_input[] = {
	{ WHEEL, WHEEL_MENU, playlist_back },
	{ KEY, KEY_LEFT, playlist_back },
	{ KEY, KEY_X, playlist_back },
	{ WHEEL, WHEEL_RIGHT, playlist_down },
	{ KEY, KEY_DOWN, playlist_down },
	{ WHEEL, WHEEL_LEFT, playlist_up },
	{ KEY, KEY_UP, playlist_up},
	{ WHEEL, WHEEL_CENTER, playlist_center },
	{ KEY, KEY_RIGHT, playlist_center },
	{ END, 0, NULL }
};
struct input_handle *playlist_input_p = playlist_input;

static struct input_handle playlists_input[] = {
	{ WHEEL, WHEEL_MENU, playlists_back },
	{ KEY, KEY_LEFT, playlists_back },
	{ KEY, KEY_X, playlists_back },
	{ WHEEL, WHEEL_RIGHT, playlists_down },
	{ KEY, KEY_DOWN, playlists_down },
	{ WHEEL, WHEEL_LEFT, playlists_up },
	{ KEY, KEY_UP, playlists_up},
	{ WHEEL, WHEEL_CENTER, playlists_center },
	{ KEY, KEY_RIGHT, playlists_center },
	{ END, 0, NULL }
};
struct input_handle *playlists_input_p = playlists_input;

void input_sound_finished(void) {
	sound_finishing();

	if(playfile != NULL) {
		sound_stop();

		fclose(playfile);
		playfile = NULL;
	}

	if(PLAYLIST->current >= PLAYLIST->size - 1)
		PLAYLIST->current = 0;
	else
		PLAYLIST->current++;

	currentmedia = PLAYLIST->list[PLAYLIST->current];

	screen_nowplaying_reset();

	playfile = fopen(currentmedia->path, "r");

	sound_setup(playfile, currentmedia->format);
	sound_playpause();
}

void input_nowplaying_setup(int pos, const char *text) {
	input_abort = 1;

	nowplaying_last_selected = pos;
	nowplaying_last_menu = screen_menu;

	screen_type = NOW_PLAYING;
}

void input_musicmenu_setup(int pos, const char *text) {
	input_abort = 1;

	music_last_selected = pos;

	screen_musicmenu();
}

void input_songs_setup(int pos, const char *text) {
	input_abort = 1;

	playlist_last_selected = pos;
	playlist_last_menu = screen_menu;

//	screen_playlistmenu(playlists.p[0]);
	input_playlist_setup(0, "Songs");
}

void input_playlist_setup(int pos, const char *text) {
	input_abort = 1;

	playlist_last_selected = pos;
	playlist_last_menu = screen_menu;

	screen_playlistmenu(playlists.p[pos]);
}

void input_playlists_setup(int pos, const char *text) {
	input_abort = 1;

	playlists_last_selected = pos;

	screen_playlistsmenu();
}

void input_playlist(int pos, const char *text) {
	if(playlists.current != playlists_playlist || PLAYLIST->current != pos) {
		playlists.current = playlists_playlist;

		PLAYLIST->current = pos;

		currentmedia = PLAYLIST->list[pos];

		menu_play();
	}

	input_nowplaying_setup(pos, text);
}

void input_playlists(int pos, const char *text) {
	if(playlists.current != pos)
		playlists_playlist = pos;

	input_playlist_setup(pos, text);
}

static void menu_down(void) {
	if(screen_menu_selected < screen_menu_items - 1)
		screen_menu_selected++;
}

static void menu_up(void) {
	if(screen_menu_selected > 0)
		screen_menu_selected--;
}

static void menu_center(void) {
	if(currentmenu[screen_menu_selected].callback != NULL)
		currentmenu[screen_menu_selected].callback(screen_menu_selected, currentmenu[screen_menu_selected].label);
}

static void menu_stop(void) {
	if(playfile != NULL) {
		sound_stop();

		fclose(playfile);
		playfile = NULL;
	}
}

static void menu_play(void) {
	if(playfile != NULL) {
		sound_stop();
		fclose(playfile);
	}

	playfile = fopen(currentmedia->path, "r");

	sound_setup(playfile, currentmedia->format);
	sound_playpause();
}

static void menu_next(void) {
	if(state == STOPPED) {
		if(PLAYLIST->current == PLAYLIST->size - 1)
			PLAYLIST->current = 0;
		else
			PLAYLIST->current++;

		currentmedia = PLAYLIST->list[PLAYLIST->current];
	} else if(state == PLAYING) {
		menu_stop();

		if(PLAYLIST->current == PLAYLIST->size - 1)
			PLAYLIST->current = 0;
		else
			PLAYLIST->current++;

		currentmedia = PLAYLIST->list[PLAYLIST->current];

		menu_play();
	}

	screen_nowplaying_reset();
}

static void menu_prev(void) {
	if(state == STOPPED) {
		if(PLAYLIST->current == 0)
			PLAYLIST->current = PLAYLIST->size - 1;
		else
			PLAYLIST->current--;

		currentmedia = PLAYLIST->list[PLAYLIST->current];
	} else if(state == PLAYING) {
		menu_stop();

		if(PLAYLIST->current == 0)
			PLAYLIST->current = PLAYLIST->size - 1;
		else
			PLAYLIST->current--;

		currentmedia = PLAYLIST->list[PLAYLIST->current];

		menu_play();
	}

	screen_nowplaying_reset();
}

static void nowplaying_back(void) {
	input_abort = 1;

	switch(nowplaying_last_menu) {
		case MUSIC:
			screen_musicmenu();
			break;

		case LIST:
			screen_playlistmenu(PLAYLIST);
			break;

		default:
		case MAIN:
			screen_mainmenu();
			break;
	}


	screen_menu_selected = nowplaying_last_selected;
}

static void nowplaying_next(void) {
	if(state == PAUSED)
		menu_stop();

	menu_next();
}

static void nowplaying_prev(void) {
	if(state == PAUSED)
		menu_stop();

	menu_prev();
}

static void nowplaying_playpause(void) {
	if(state == PLAYING || state == PAUSED)
		sound_playpause();
	else if(state == STOPPED)
		menu_play();
}

static void music_back(void) {
	input_abort = 1;

	screen_mainmenu();

	screen_menu_selected = music_last_selected;
}

static void music_down(void) {
	if(screen_menu_selected < screen_menu_items - 1)
		screen_menu_selected++;
}

static void music_up(void) {
	if(screen_menu_selected > 0)
		screen_menu_selected--;
}

static void music_center(void) {
	if(currentmenu[screen_menu_selected].callback != NULL)
		currentmenu[screen_menu_selected].callback(screen_menu_selected, currentmenu[screen_menu_selected].label);
}

static void playlist_back(void) {
	input_abort = 1;

	switch(playlist_last_menu) {
		case MUSIC:
			screen_musicmenu();
			break;

		case LISTS:
			screen_playlistsmenu();
			break;

		default:
		case MAIN:
			screen_mainmenu();
			break;
	}


	screen_menu_selected = playlist_last_selected;
}

static void playlist_up(void) {
	if(screen_menu_selected > 0)
		screen_menu_selected--;
}

static void playlist_down(void) {
	if(screen_menu_selected < screen_menu_items - 1)
		screen_menu_selected++;
}

static void playlist_center(void) {
	if(currentmenu[screen_menu_selected].callback != NULL)
		currentmenu[screen_menu_selected].callback(screen_menu_selected, currentmenu[screen_menu_selected].label);
}

static void playlists_back(void) {
	input_abort = 1;

	screen_musicmenu();

	screen_menu_selected = playlists_last_selected;
}

static void playlists_up(void) {
	if(screen_menu_selected > 0)
		screen_menu_selected--;
}

static void playlists_down(void) {
	if(screen_menu_selected < screen_menu_items - 1)
		screen_menu_selected++;
}

static void playlists_center(void) {
	if(currentmenu[screen_menu_selected].callback != NULL)
		currentmenu[screen_menu_selected].callback(screen_menu_selected, currentmenu[screen_menu_selected].label);
}

void input_handleinput(void) {
	u16 wheel;
	uint32 keys;
	int i;

	wheel = IPC2->wheel_status;
	keys = keysDown();

	input_abort = 0;
	for(i = 0; input_abort == 0 && currenthandle[i].type != END; i++) {
		switch(currenthandle[i].type) {
			case WHEEL:
				if(wheel & currenthandle[i].key)
					currenthandle[i].callback();
				break;

			case KEY:
				if(keys & currenthandle[i].key)
					currenthandle[i].callback();
				break;

			default:
				break;
		}
	}
}
